# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

""" Module events

Defines the events and a timer class.


"""

import sys
import time
import traceback
import weakref


# the CallableObject class is now equal

class CallableObject(object):
    """ CallableObject(callable)
    
    A class to hold a callable. If it is a plain function, its reference
    is held (because it might be a closure). If it is a method, we keep
    the function name and a weak reference to the object. In this way,
    having for instance a signal bound to a method, the object is not
    prevented from being cleaned up.
    
    """
    __slots__ = ['_ob', '_func']  # Use __slots__ to reduce memory footprint
    
    def __init__(self, c):
        
        # Check
        if not hasattr(c, '__call__'):
            raise ValueError('Error: given callback is not callable.')
        
        # Store funcion and object
        if hasattr(c, '__self__'):
            # Method, store object and method name
            self._ob = weakref.ref(c.__self__)
            self._func = c.__func__.__name__
        elif hasattr(c, 'im_self'):
            # Method in older Python
            self._ob = weakref.ref(c.im_self)
            self._func = c.im_func.__name__
        else:
            # Plain function
            self._func = c
            self._ob = None
    
    def isdead(self):
        """ Get whether the weak ref is dead.
        """
        if self._ob:
            # Method
            return self._ob() is None
        else:
            return False
    
    def compare(self, other):
        """ compare this instance with another.
        """
        if self._ob and other._ob:
            return (self._ob() is other._ob()) and (self._func == other._func)
        elif not (self._ob or other._ob):
            return self._func == other._func
        else:
            return False
    
    def __str__(self):
        return self._func.__str__()
    
    def call(self, *args, **kwargs):
        """ call(*args, **kwargs)
        Call the callable. Exceptions are caught and printed.
        """
        if self.isdead():
            return
        
        # Get function
        try:
            if self._ob:
                func = getattr(self._ob(), self._func)
            else:
                func = self._func
        except Exception:
            return
        
        # Call it
        return func(*args, **kwargs)


class BaseEvent:
    """ The BaseEvent is the simplest type of event.
    
    The purpose of the event class is to provide a way to bind/unbind
    to events and to fire them. At the same time, it is the place where
    the properties of the event are stored (such mouse location, key
    being pressed, ...).
    
    In Qt speak, an event is a combination of an event and a signal.
    Multiple callbacks can be registered to an event (signal-paradigm).
    A callback may chose to override previously registered callbacks
    by using setHandled(). If there are no handlers, or if the event
    is explicitly ignored (using the Ignore method) by *all* handlers,
    the event is propagated to the parent object (event paradigm).
    
    To register or unregister a callback, use the Bind and Unbind methods
    When fired, all handlers that are bind to this event are called,
    until the event is set as handled. The handlers are called with the
    event object as an argument. The event.owner provides a reference of
    what wobject/wibject fired the event.
    
    """
    
    _PROPAGATE = False
    
    def __init__(self, owner):
        self._owner = weakref.ref(owner)
        self._handlers = []
        
        # For propagation to parent and sibling handlers
        self._accepted = True
        self._isHandled = False
        
        # Properties
        self._modifiers = ()
    
    
    @property
    def owner(self):
        """ The object that this event belongs to.
        """
        return self._owner()


    @property
    def type(self):
        """ The type (__class__) of this event.
        """
        return self.__class__
    
    @property
    def eventName(self):
        """ The name of this event.
        """
        className = self.__class__.__name__
        return className[0].lower() + className[1:]
    
    @property
    def hasHandlers(self):
        """ Get whether this event has handlers registered to it.
        """
        return bool(self._handlers)
    
    def Accept(self):
        """ Accept()
        Accept the event, preventing it from being propagated to
        the parent object. When an event handler is called, the event
        is accepted by default.
        """
        self._accepted = True
    
    def Ignore(self):
        """ Ignore()
        Clears the accept flag parameter of the event object. Clearing the
        accept parameter indicates that the event receiver does not want
        the event. If none of the handlers for this event do not want it,
        the event might be propagated to the parent object.
        """
        self._accepted = False
    
    def SetHandled(self, isHandled=True):
        """ SetHandled(isHandled=True)
        Mark the event as handled, preventing any subsequent handlers
        from being called. Use this in a custom handler to override the
        existing  handler. The same behavior is obtained when the handler
        returns True.
        """
        self._isHandled = bool(isHandled)
    
    
    def Bind(self, func):
        """ Bind(func)
        
        Add an eventhandler to this event.
        
        The callback/handler (func) must be a callable. It is called
        with one argument: the event instance, which contains the mouse
        location for the mouse event and the keycode for the key event.
        
        """
        
        # check
        if not hasattr(func, '__call__'):
            raise ValueError('Warning: can only bind callables.')
        
        # make callable object
        cnew = CallableObject(func)
        
        # check -> warn
        for c in self._handlers:
            if cnew.compare(c):
                print("Warning: handler %s already present for %s" %(func, self))
                return
        
        # add the handler
        self._handlers.append( cnew )
        
        self._UpdateOwner()
    

    def Unbind(self, func=None):
        """ Unbind(func=None)
        
        Unsubscribe a handler, If func is None, remove all handlers.
        
        """
        if func is None:
            self._handlers[:] = []
        else:
            cref = CallableObject(func)
            for c in [c for c in self._handlers]:
                # remove if callable matches func or object is destroyed
                if c.compare(cref) or c.isdead():
                    self._handlers.remove( c )
        
        self._UpdateOwner()
    
    
    def Fire(self, *eventArgs):
        """ Fire(*eventArgs)
        
        Fire the event, calling all functions that are bound
        to it, untill the event is handled (a handler returns True).
        
        If no handlers are present or if the event is explicitly ignored,
        the event is propagated to the owners parent. This only apples
        to events for which this is approprioate and only if eventArgs
        are given.
        
        """
        self._isFired = True # Flag used by BaseFigure
        
        # If event args given, set first
        if eventArgs:
            self.Set(*eventArgs)
        
        # remove dead weakrefs
        for c in [c for c in self._handlers]:
            if c.isdead():
                self._handlers.remove( c )
                self._UpdateOwner()
        
        # get list of callable functions
        L = self._handlers
        
        # Init handled and accepted
        # In the event, the event is accepted if one of the handlers
        # accepted (i.e. did not ignore) it
        accepted = False
        handled = False
        
        # call event handlers. Call last added first! ...
        func = None
        for func in reversed( L ):
            try:
                # Init handled and accepted for next call
                self._isHandled = False
                self._accepted = True # Default is accepted if handled
                # Do the call and determine if handled
                res = func.call(self)
                handled = res or self._isHandled
            except Exception:
                # On error, notify, allow postmortem debugging and also break
                self._HandleExceptionInCallback(func)
            # Handle accepted and handled
            accepted = accepted or self._accepted
            if handled:
                break
        
        if self._PROPAGATE and eventArgs and not accepted:
            # The event is not properly handled yet, try the parent object
            ob = self.owner
            if (ob is not None) and (ob.parent is not None):
                # Try getting the event object with the same name
                parentEvent = getattr(ob.parent, self.eventName, None)
                if parentEvent is not None:
                    # Fire parent events with given event arguments
                    parentEvent.Fire(*eventArgs)
    
    
    def _HandleExceptionInCallback(self, func):
        """ Print a nice message of what went wrong and also set sys.last_*
        for postmortem debugging.
        """
        # get easier func name
        s = str(func)
        i = s.find("function")
        if i<0:
            i = s.find("method")
        if i>= 0:
            i1 = s.find(" ",i+1)
            i2 = s.find(" ",i1+1)
            if i1>=0 and i2>=0:
                s = s[i1+1:i2]
        # get traceback and store
        type, value, tb = sys.exc_info()
        sys.last_type = type
        sys.last_value = value
        sys.last_traceback = tb
        # Show traceback
        tblist = traceback.extract_tb(tb)
        list = traceback.format_list(tblist[2:]) # remove "Fire"
        list.extend( traceback.format_exception_only(type, value) )
        # print
        print("ERROR calling '%s':" % s)
        tmp = ""
        for i in list:
            tmp += i
        print(tmp)
    
    
    def _UpdateOwner(self):
        """ Update the hitTest property of the object.
        """
        owner = self.owner
        if owner and hasattr(owner, '_testWhetherShouldDrawShape'):
            owner._testWhetherShouldDrawShape()
    
    # Event properties:
    
    @property
    def modifiers(self):
        """ The modifier keys active when the event occurs.
        """
        return self._modifiers
    
    # Event classes should overload this
    def Set(self, modifiers=()):
        """ Set(modifiers)
        
        Set the event properties before firing it. In the base event
        the only property is the modifiers state, a tuple of the
        modifier keys currently pressed.
        
        """
        self._modifiers = modifiers
    


class MouseEvent(BaseEvent):
    """ MouseEvent(owner)
    
    A MouseEvent is an event for things that happen  with the mouse.
    
    """
    
    def __init__(self, owner):
        BaseEvent.__init__(self, owner)
        self._x, self._y = 0, 0
        self._x2d, self._y2d = 0, 0
        self._but = 0
    
    
    def Set(self, absx, absy, but, modifiers=()):
        """ Set(absx, absy, but, modifiers=())
        
        Set the event properties before firing it.
        
        """
        BaseEvent.Set(self, modifiers)
        
        # Set properties we can alway set
        self._absx = absx
        self._absy = absy
        self._but = but
        
        # Init other properties
        self._x = absx
        self._y = absy
        self._x2d = 0
        self._y2d = 0
        
        # Try getting more information on the owning object
        owner = self._owner()
        if owner:
            
            # Can we Set the event at all?
            if owner._destroyed:
                return
            
            # Determine axes (for Wobjects)
            axes = None
            if hasattr(owner, 'GetAxes'):
                axes = owner.GetAxes()
                if not axes:
                    return
                if not hasattr(axes, '_cameras'):
                    axes = None # For example a legend
            
            if hasattr(owner, 'position'):
                # A Wibject: use relative coordinates if not a figure
                if owner.parent:
                    self._x -= owner.position.absLeft
                    self._y -= owner.position.absTop
            elif axes:
                # A Wobject: use axes coordinates
                self._x -= axes.position.absLeft
                self._y -= axes.position.absTop
            
            if axes or hasattr(owner, '_cameras'):
                # Also give 2D coordinates
                if axes:
                    cam = axes._cameras['TwoDCamera']
                else:
                    cam = owner._cameras['TwoDCamera']
                if owner.parent: # or screen to world cannot be calculated
                    self._x2d, self._y2d = cam.ScreenToWorld((self._x, self._y))
    
    
    @property
    def absx(self):
        """ The absolute x position in screen coordinates when the event
        happened.
        """
        return self._absx
    
    @property
    def absy(self):
        """ The absolute y position in screen coordinates when the event
        happened.
        """
        return self._absy
    
    @property
    def x(self):
        """ The x position in screen coordinates relative to the owning object
        when the event happened. (For Wobjects, relative to the Axes.)
        """
        return self._x
    
    @property
    def y(self):
        """ The y position in screen coordinates relative to the owning object
        when the event happened. (For Wobjects, relative to the Axes.)
        """
        return self._y
    
    @property
    def x2d(self):
        """ The x position in 2D world coordinates when the event happened.
        This is only valid when the used camera is 2D.
        """
        return self._x2d
    
    @property
    def y2d(self):
        """ The y position in 2D world coordinates when the event happened.
        This is only valid when the used camera is 2D.
        """
        return self._y2d
    
    @property
    def button(self):
        """ The The mouse button that was pressed, 0=none, 1=left, 2=right.
        """
        return self._but


class KeyEvent(BaseEvent):
    """ KeyEvent(owner)
    
    A KeyEvent event is an event for things that happen with the keyboard.
    
    """
    
    def __init__(self, owner):
        BaseEvent.__init__(self, owner)
        self._key = 0
        self._text = ''
    
    def Set(self, key, text='', modifiers=()):
        """ Set(key, text='')
        
        Set the event properties before firing it.
        
        """
        BaseEvent.Set(self, modifiers)
        self._key = key
        self._text = text
    
    @property
    def key(self):
        """ The integer keycode of the key.
        """
        return self._key
    
    @property
    def text(self):
        """ The text that the key represents (if available).
        """
        return self._text


## Specific events for all objects
# Make classes for each specific event that is standard for all object,
# to help introspection.

class EventMouseDown(MouseEvent):
    """ EventMouseDown(owner)
    
    Fired when the mouse is pressed down on this object, or on any of its
    children and not being handled.
    (Also fired the first click of a double click.)
    
    """
    _PROPAGATE = True
    def Fire(self, *eventArgs):
        if self.owner:
            self.owner._mousePressedDown = True
        MouseEvent.Fire(self, *eventArgs)

class EventMouseUp(MouseEvent):
    """ EventMouseUp(owner)
    
    Fired when the mouse is released after having been clicked down
    on this object (even if the mouse is now not over the object). (Also
    fired on the first click of a double click.)
    
    """
    _PROPAGATE = False
    def Fire(self, *eventArgs):
        if self.owner:
            self.owner._mousePressedDown = False
        MouseEvent.Fire(self, *eventArgs)

class EventDoubleClick(MouseEvent):
    """ EventDoubleClick(owner)
    
    Fired when the mouse is double-clicked on this object, or on any of its
    children and not being handled.
    
    """
    _PROPAGATE = True

class EventEnter(MouseEvent):
    """ EventEnter(owner)
    
    Fired when the mouse enters this object or any of its children.
    
    """
    _PROPAGATE = False

class EventLeave(MouseEvent):
    """ EventLeave(owner)
    
    Fired when the mouse was previously over the object or any of its
    children, and is now not.
    
    """
    _PROPAGATE = False

class EventMotion(MouseEvent):
    """ EventMotion(owner)
    
    Fired when the mouse is moved over the object, or over any of its
    children and not being handled. This event is also always called
    when the mouse is pressed down on the object.
    """
    _PROPAGATE = False

class EventScroll(MouseEvent):
    """ EventScroll(owner)
    
    Fired when the scroll wheel is used while over the object, or on any of its
    children and not being handled. The value of the button property is
    undefined for this event. The amount of scrolling is available as the
    horizontalSteps and verticalSteps properties.
    
    """
    _PROPAGATE = True
    
    def __init__(self, owner):
        MouseEvent.__init__(self, owner)
        self._horizontalSteps = 0.0
        self._verticalSteps = 0.0
    
    def Set(self, absx, absy, horizontalSteps, verticalSteps, modifiers=()):
        """ Set(absx, absy, horizontalSteps, verticalSteps, modifiers=())
        """
        MouseEvent.Set(self, absx, absy, 0, modifiers) # Button is 0
        self._horizontalSteps = horizontalSteps
        self._verticalSteps = verticalSteps

    @property
    def horizontalSteps(self):
        """ Get the amount of scrolling in the horizontal direction
        (can be positive or negative) .
        """
        return self._horizontalSteps
    
    @property
    def verticalSteps(self):
        """ Get the amount of scrolling in the vertical direction
        (can be positive or negative) .
        """
        return self._verticalSteps


class EventKeyDown(KeyEvent):
    """ EventKeyDown(owner)
    
    Fired when a key is pressed down while the figure is active.
    
    """
    _PROPAGATE = True

class EventKeyUp(KeyEvent):
    """ EventKeyUp(owner)
    
    Fired when a key is released while the figure is active.
    
    """
    _PROPAGATE = True

## Only for wibjects

class EventPosition(BaseEvent):
    """ EventPosition(owner)
    
    Fired when the position (or size) of this wibject changes.
    
    """
    _PROPAGATE = False
    

## Processing events + timers

# For callLater function
_callLater_callables = {}

def processVisvisEvents():
    """ processVisvisEvents()
    
    Process all visvis events. Checks the status of all timers
    and fires the ones that need to be fired. This method
    needs to be called every now and then.
    
    All backends implement a timer that periodically calls this function.
    
    To keep a figure responsive while running, periodically call
    Figure.DrawNow() or vv.processEvents().
    
    """
    Timer._TestAllTimers()


class Timer(BaseEvent):
    """ Timer(owner, interval=1000, oneshot=True)
    
    Timer class. You can bind callbacks to the timer. The timer is
    fired when it runs out of time. You can do one-shot runs and
    continuous runs.
    
    Setting timer.nolag to True will prevent the timer from falling
    behind. If the previous Fire() was a bit too late the next Fire
    will take place sooner. This will make that at an interval of
    1000, 3600 events will have been fired in one hour.
    
    """
    
    _timers = []
    
    def __init__(self, owner, interval=1000, oneshot=True):
        # register
        Timer._timers.append( weakref.ref(self) )
        
        # store info being an event
        self._owner = weakref.ref(owner)
        self._handlers = []
        
        # store Timer specific properties
        self.interval = interval
        self.oneshot = oneshot
        self.nolag = False
        self._running = False
        self._timestamp = 0


    def Start(self, interval=None, oneshot=None):
        """ Start(interval=None, oneshot=None)
        
        Start the timer. If interval end oneshot are not given,
        their current values are used.
        
        """
        # set properties?
        if interval is not None:
            self.interval = interval
        if oneshot is not None:
            self.oneshot = oneshot
        
        # put on
        self._running = True
        self._timestamp = time.time() + (self.interval/1000.0)
    
    
    def Stop(self):
        """ Stop()
        
        Stop the timer from running.
        
        """
        self._running = False
    
    
    def Destroy(self):
        """ Destroy()
        
        Destroy the timer, preventing it from ever fyring again.
        
        """
        self.Stop()
        tmp = weakref.ref(self)
        if tmp in Timer._timers:
            Timer._timers.remove(tmp)
    
    @property
    def isRunning(self):
        """ Get whether the timer is running.
        """
        return self._running
    
    
    @classmethod
    def _TestAllTimers(self):
        """ Method used to test all timers whether they should be
        fired. If so, it fires them.
        """
        
        # test calLaters first
        for calltime in [key for key in _callLater_callables.keys()]:
            if calltime < time.time():
                callable, args, kwargs = _callLater_callables.pop(calltime)
                callable(*args, **kwargs)
        
        timersToRemove = []
        for timerRef in Timer._timers:
            timer = timerRef()
            
            # check if timer exists, otherwise remove.
            if timer is None:
                timersToRemove.append(timerRef)
                continue
            
            # is it on?
            if not timer._running:
                continue
            
            # has the time passed yet?
            if time.time() > timer._timestamp:
                timer.Fire()
            else:
                continue
            
            # do we need to stop it?
            if timer.oneshot:
                timer._running = False
            else:
                if timer.nolag:
                    timer._timestamp += (timer.interval/1000.0)
                else:
                    timer._timestamp = time.time() + (timer.interval/1000.0)
        
        # clean up any dead references
        for timerRef in timersToRemove:
            try:
                Timer._timers.remove(timerRef)
            except Exception:
                pass

class App:
    """ App()
    
    The App class wraps a GUI backend  with a simple interface that is
    the same for all backends. It can be used to start the GUI toolkit's
    main-loop, or process all pending events.
    
    To obtain an instance of this class, the user should call vv.use().
    
    """
    
    def __repr__(self):
        name = self.GetBackendName()
        if not name:
            name = 'undefined'
        return '<Visvis app that wraps the %s GUI toolkit>' % name
    
    
    def GetBackendName(self):
        """ GetBackendName()
        
        Get the name of the GUI backend that this app wraps.
        
        """
        if hasattr(self, '_backend'):
            return self._backend
        else:
            return ''
    
    
    def GetFigureClass(self):
        """ GetFigureClass()
        
        Get the Figure class for this backend.
        
        """
        backendModule = sys.modules[self.__module__]
        return backendModule.Figure
    
    
    def Create(self):
        """ Create()
        
        Create the native application object. When embedding visvis in an
        application, call this method before instantiating the main window.
        
        """
        # Make sure the app exists
        self._GetNativeApp()
    
    
    def ProcessEvents(self):
        """ ProcessEvents()
        
        Process all pending GUI events. This should be done regularly
        to keep the visualization interactive and to keep the visvis
        event system running.
        
        When using IPython or IEP with the right settings, GUI events
        will be processed automatically. However, in a running script,
        this is not the case; be then regularly calling this method,
        the figures will stay responsive.
        
        """
        self._ProcessEvents()
    
    
    def Run(self):
        """ Run()
        
        Enter the native GUI event loop.
        
        """
        self._Run()
    
    
    # Implement these methods. Be using this redirection scheme, we keep
    # the documentation intact.
    def _GetNativeApp(self):
        raise NotImplementedError()
    
    def _ProcessEvents(self):
        raise NotImplementedError()
    
    def _Run(self):
        raise NotImplementedError()



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import visvis as vv
 
def colorbar(axes=None):
    """ colorbar(axes=None)
    
    Attach a colorbar to the given axes (or the current axes if
    not given). The reference to the colorbar instance is returned.
    Also see the vv.ColormapEditor wibject.
    
    """
    
    if axes is None:
        axes = vv.gca()
    
    return vv.Colorbar(axes)


if __name__ == '__main__':
    # Create and show grayscale image, using a colormap
    im = vv.imread('astronaut.png')[:,:,0]
    t = vv.imshow(im)
    t.colormap = vv.CM_COPPER
    # Show how the values map to colors
    vv.colorbar()



# -*- coding: utf-8 -*-
# flake8: noqa
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

# This file is a placeholder. The backend system of visvis assumes one
# backend per module. Since the PyQt and PySide backends share almost
# all code, its nonsense to duplicate it. Instead we use these small
# placeholder modules that import qtcommon.

import visvis

# Check if qtlib is already set
if visvis.backends.qtlib not in [None, '', 'pyside']:
    raise ImportError('Cannot import PySide because Qt was already loaded from "%s".' %
                        visvis.backends.qtlib)

# Set qtlib so that the qtcommon module knows how it should import Qt.
visvis.backends.qtlib = 'pyside'

# Load backend from qtcommon
from visvis.backends.qtcommon import Figure, newFigure, App, app



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import visvis as vv

# Try importing imageio
imageio = None

try:
    import imageio
except ImportError:
    pass


def imwrite(filename, image, format=None):
    """ imwrite(filename, image, format=None)
    
    Write image (numpy array) to file, requires imageio.
    
    Parameters
    ----------
    filename : string
        The name of the file to store the screenshot to. If filename is None,
        the interpolated image is returned as a numpy array.
    image : numpy array
        The image to write.
    format : string
        The format for the image to be saved in. If not given, the
        format is deduced from the filename.
    
    Notes
    -----
      * For floating point images, 0 is considered black and 1 is white.
      * For integer types, 0 is considered black and 255 is white.
    
    """
    
    if imageio is None:
        raise RuntimeError("visvis.imwrite requires the imageio package.")
    
    imageio.imwrite(filename, image, format)


if __name__ == '__main__':
    im = vv.imread('astronaut.png')
    vv.imwrite('astronaut_new.jpg', im)



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

""" Module stl

This module produces functionality to read and write STL files
(binary and ascii).

http://en.wikipedia.org/wiki/STL_%28file_format%29

STL is intended to describe solids; the mesh represents the outer boundary.
Therefore it is important that the faces face the right way (to the outside).

The format specifies a normal for each face, therefore the normal is
actually redundant information. We do not use the normal when reading,
and write a dummy normal on writing. The format also unly supports triangular
faces.

The classes are written with compatibility of Python3 in mind.

"""

import visvis as vv
import numpy as np
import struct


class StlReader(object):
    
    def __init__(self, f):
        self._f = f
    
    @classmethod
    def read(cls, fname, check=False):
        """ read(fname, check=False)
        
        This classmethod is the entry point for reading STL files.
        
        Parameters
        ----------
        fname : string
            The name of the file to read.
        check : bool
            If check is True and the file is in ascii, some checks to the
            integrity of the file are done (which is a bit slower).
        
        """
        
        # Open file
        f = open(fname, 'rb')
        try:
            
            # Read whitespace
            while not f.read(1).strip():
                pass
            # Go one back
            f.seek(-1,1)
            
            # Determine ascii or binary
            data = f.read(5).decode('ascii', 'ignore')
            if data == 'solid':
                # Pop rest of line (i.e. the name) and get reader object
                f.readline()
                reader = StlAsciiReader(f)
            else:
                # Pop rest if header (header has 80 chars) and get reader object
                f.read(75)
                reader = StlBinReader(f)
            
            # Read all
            vertices = vv.Pointset(3)
            while True:
                reader.readFace(vertices, check)
            
        except EOFError:
            pass
        finally:
            f.close()
        
        # Done
        return vv.BaseMesh(vertices)


class StlWriter(object):
    
    def __init__(self, f):
        self._f = f
    
    
    @classmethod
    def _convertQuadsToTriangles(cls, mesh):
        """ STL only allows triangles, therefore we need a method
        to convert triangles to quads.
        """
        if mesh._faces is not None:
            faces = mesh._GetFaces()
            vertices = mesh._vertices
            vv1 = vertices[faces[:,0]]
            vv2 = vertices[faces[:,1]]
            vv3 = vertices[faces[:,2]]
            vv4 = vertices[faces[:,3]]
        else:
            vv1 = mesh._vertices[0::4]
            vv2 = mesh._vertices[1::4]
            vv3 = mesh._vertices[2::4]
            vv4 = mesh._vertices[3::4]
        
        tri1 = vv.Pointset(vv1)
        tri2 = vv.Pointset(vv2)
        tri3 = vv.Pointset(vv3)
        #
        tri1.extend(vv.Pointset(vv3))
        tri2.extend(vv.Pointset(vv4))
        tri3.extend(vv.Pointset(vv1))
        
        return tri1, tri2, tri3
    
    
    @classmethod
    def write(cls, fname, mesh, name='', bin=True):
        """ write(fname, mesh, name='', bin=True)
        
        This classmethod is the entry point for writing mesh data to STL.
        
        Parameters
        ----------
        fname : string
            The filename to write to.
        mesh : Mesh data
            Can be vv.BaseMesh, vv.Pointset, or np.ndarray.
        name : string
            The name of the object (e.g. 'teapot')
        bin : bool
            Whether to write binary, which is much more compact then ascii.
        
        """
        
        if isinstance(mesh, vv.BaseMesh):
            if mesh._verticesPerFace != 3:
                vv1, vv2, vv3 = cls._convertQuadsToTriangles(mesh)
            elif mesh._faces is not None:
                faces = mesh._GetFaces()
                vertices = mesh._vertices
                # Get vertex per faces index (for all faces)
                vv1 = vertices[faces[:,0]]
                vv2 = vertices[faces[:,1]]
                vv3 = vertices[faces[:,2]]
            else:
                vv1 = mesh._vertices[0::3]
                vv2 = mesh._vertices[1::3]
                vv3 = mesh._vertices[2::3]
        elif vv.utils.pypoints.is_Pointset(mesh):
            if mesh.ndim != 3:
                raise ValueError('Mesh vertices must be 3D.')
            vv1 = mesh[0::3]
            vv2 = mesh[1::3]
            vv3 = mesh[2::3]
        elif isinstance(mesh, np.ndarray):
            if mesh.shape[1] != 3:
                raise ValueError('Mesh vertices must be 3D.')
            vv1 = mesh[0::3]
            vv2 = mesh[1::3]
            vv3 = mesh[2::3]
        else:
            raise ValueError('Unknown type for mesh vertices.')
        
        
        # Open file
        f = open(fname, 'wb')
        try:
            # Get writer and write head
            if bin:
                writer = StlBinWriter(f)
                f.write(struct.pack('<B', 0)*80)
                f.write(struct.pack('<I', len(vv1)))
            else:
                writer = StlAsciiWriter(f)
                writer.writeLine('solid %s' % name)
            # Write vertices
            for i in range(len(vv1)):
                writer.writeFace(vv1[i], vv2[i], vv3[i])
            # Write end
            if not bin:
                writer.writeLine('endsolid %s' % name)
        except EOFError:
            pass
        finally:
            f.close()


class StlAsciiReader(StlReader):
        
    def readline(self):
        """ Simple readLine method that strips whitespace and skips
        empty lines.
        """
        line = ''
        while not line:
            line = self._f.readline().decode('ascii', 'ignore').strip()
        return line
    
    
    def readFace(self, vertices, check=False):
        """ readFace(vertices, check=False)
        
        Read a face (three vertices) from the file. The normal is ignored;
        we will calculate it ourselves.
        
        Info: http://en.wikipedia.org/wiki/STL_%28file_format%29
        
        What a face in a file looks like
        --------------------------------
        facet normal ni nj nk
        outer loop
        vertex v1x v1y v1z
        vertex v2x v2y v2z
        vertex v3x v3y v3z
        endloop
        endfacet
        
        """
        
        # Read normal and identifier
        line_normal = self.readline()
        if line_normal.startswith('endsolid'):
            raise EOFError() # Finished
        line_begin_loop = self.readline()
        
        # Read 3 vertices
        for i in range(3):
            line_vertex = self.readline()
            numbers = [num for num in line_vertex.split(' ')]
            numbers = [float(num) for num in numbers[1:] if num]
            vertices.append(*numbers)
        
        # Read two more identifiers
        line_end_loop = self.readline()
        line_end_facet = self.readline()
        
        # Test?
        if check:
            if not line_normal.startswith('facet normal'):
                print('Warning: expected facet normal.')
            if line_begin_loop != "outer loop":
                print('Warning: expected outer loop.')
            if line_end_loop != "endloop":
                print('Warning: expected endloop.')
            if line_end_facet != "endfacet":
                print('Warning: expected endfacet.')


class StlAsciiWriter(StlWriter):
    
    def writeLine(self, text):
        """ Simple writeLine function to write a line of code to the file.
        The encoding is done here, and a newline character is added.
        """
        text += '\n'
        self._f.write(text.encode('ascii'))
    
    
    def writeFace(self, v1, v2, v3):
        """ writeFace(v1, v2, v3)
        
        Write the three vertices that make up a face.
        
        """
        
        # Write dummy normal and start-of-3-vertices-tag
        self.writeLine('facet normal 0.0E+00 0.0E+00 0.0E+00')
        self.writeLine('outer loop')
        
        # Write three vertices
        for v in [v1, v2, v3]:
            self.writeLine('vertex %E %E %E' % (v[0], v[1], v[2]))
        
        # Finish facet
        self.writeLine('endloop')
        self.writeLine('endfacet')


class StlBinReader(StlReader):
    
    def __init__(self, f):
        StlReader.__init__(self, f)
        
        # Get number of faces in this file
        self._n, = struct.unpack('<I', f.read(4))
        self._count = 0
    
    
    def readFace(self, vertices, check=False):
        """ readFace(vertices, check=False)
        
        Read a face (three vertices) from the file. The normal is ignored;
        we will calculate it ourselves.
        
        Info: http://en.wikipedia.org/wiki/STL_%28file_format%29
        
        A face consists of 12 32bit float numbers and a short int.
        The 12 numbers are the 3 vertices and the normal.
        
        """
        
        # Reached last face?
        if self._count >= self._n:
            raise EOFError()
        
        # Read data.
        # 12 32bit floating point numbers + short int used by some apps
        n = 12*4+2
        data = self._f.read(n)
        if len(data) < n:
            raise EOFError()
        
        # Convert numbers
        numbers = [struct.unpack('<f', data[i*4:(i+1)*4]) for i in range(12)]
        #attribute_byte_count = struct.unpack('<H', data[12*4:12*4+2]
        
        # Add vertices, ignore normal
        vertices.append(*numbers[3:6])
        vertices.append(*numbers[6:9])
        vertices.append(*numbers[9:12])
        
        # Increase counter
        self._count += 1
    

class StlBinWriter(StlWriter):
    
    def writeFace(self, v1, v2, v3):
        """ writeFace(v1, v2, v3)
        
        Write the three vertices that make up a face.
        
        """
        
        # Construct data
        dataList = []
        for i in range(3):
            dataList.append( struct.pack('<f', 0.0) )
        for p in v1:
            dataList.append( struct.pack('<f', p) )
        for p in v2:
            dataList.append( struct.pack('<f', p) )
        for p in v3:
            dataList.append( struct.pack('<f', p) )
        if True:
            dataList.append( struct.pack('<H', 0) )
        
        # Write data
        data = ''.encode('ascii').join(dataList)
        self._f.write(data)



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import visvis as vv

def getOpenGlInfo():
    """ getOpenGlInfo()
    
    Get information about the OpenGl version on this system.
    Returned is a tuple (version, vendor, renderer, extensions)
    
    A figure is created and removed to create an openGl context if
    this is necessary.
    
    """
    
    # Open figure first. On Windows we can try obtaining the information,
    # but I found that on Ubuntu a segfault will happen (but this might
    # very well have to do with the OpenGl drivers).
    fig = vv.figure()
    result = vv.misc.getOpenGlInfo()
    
    # Should we open a figure and try again?
    fig.Destroy()
    fig._ProcessGuiEvents() # so it can close
    
    return result

if __name__ == '__main__':
    # get info
    version, vendor, renderer, ext = vv.getOpenGlInfo()
    # print!
    print('Information about the OpenGl version on this system:\n')
    print('version:    ' + version)
    print('vendor:     ' + vendor)
    print('renderer:   ' + renderer)
    print('extensions: ' + str(len(ext.split())) + ' different extensions')



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

""" Module baseFigure

Defines the BaseFigure class, which is the root of the drawing context,
and is associated with an OpenGL context.

This module also defines a few helper classes for the Figure.

"""

import OpenGL.GL as gl

import time
import numpy as np

import visvis
#
from visvis.core import base
from visvis.core.base import DRAW_NORMAL, DRAW_FAST, DRAW_SHAPE, DRAW_SCREEN  # noqa
from visvis.core.misc import Property, PropWithDraw, DrawAfter
from visvis.core.misc import getOpenGlInfo
from visvis.core import events
#
from visvis.core.cameras import ortho
from visvis.text import BaseText
from visvis.core.line import MarkerManager
from visvis.core.axes import _BaseFigure, AxesContainer, Axes, Legend
from visvis.core.axes import _Screenshot


# a variable to indicate whether to show FPS, for testing
printFPS = False


class ObjectPickerHelper(object):
    """ ObjectPickerHelper()
    
    A simple class to help picking of the objects.
    An instance of this is attached to each figure.
    
    Picking in visvis is done using the approach proposed in chapter 14 of
    the 6th edition of the Red Book: rendering to the backbuffer using a
    particular color per object.
    
    Another possible approach is gluPickMatrix. Although the backbuffer
    approach might be a bit slower, it is easier than the pickmatrix approach
    and allows more control over what objects you want to be able to pick.
    
    """
    
    def __init__(self):
        self.bits_r, self.bits_g, self.bits_b = 8, 8, 8
        self.curid = 0
        self.screen = None  # the screenshot
    
    def GetId(self):
        """ Get an id.  """
        self.curid += 1
        return self.curid
    
    def GetColorFromId(self,id):
        # make int to be sure
        id = int(id)
        # get factors
        fr, fg, fb = 2**self.bits_r, 2**self.bits_g, 2**self.bits_b
        # get an id for each color
        idr = id // (fg*fb)
        id  = id % (fg*fb)
        idg = id // fb
        idb = id % fb
        if idr>fr:
            # this will probably never happen
            raise Exception("Your id exceededs what you can express in color!")
        # return
        return float(idr)/(fr-1), float(idg)/(fg-1), float(idb)/(fb-1)
    
    def GetIdFromColor(self,r,g,b):
        # get factors
        fr, fg, fb = 2**self.bits_r, 2**self.bits_g, 2**self.bits_b
        idr, idg, idb = int(r*fr), int(g*fg), int(b*fb)
        #idr, idg, idb = int(r), int(g), int(b)
        id =  (idr*fg + idg)*fb + idb
        return int(id)
    
    def CaptureScreen(self):
        self.screen = np.flipud( _Screenshot() )
    
    def ClearScreen(self):
        self.screen = None
    
    def GetItemsUnderMouse(self, figure):
        """ Detect over which objects the mouse is now.
        """
        # make sure screen exists
        if self.screen is None:
            self.screen = np.zeros((1,1),dtype=np.float32)
        
        # get shape and position of mouse
        shape = self.screen.shape
        x, y = figure.mousepos
        x = int(x * figure._devicePixelRatio)
        y = int(y * figure._devicePixelRatio)
         
        # get id of the object under the mouse
        if x < 0 or x >= shape[1] or y < 0 or y >= shape[0]:
            id = 0
        else:
            clr = self.screen[y, x]
            id = self.GetIdFromColor(clr[0],clr[1],clr[2])
        
        # search the object
        items = [figure]  # figure is always at the bottom
        if id:
            self._walkTree(id, figure._children, items)
            #print(id)
        
        # return result
        return items
    
    
    def AssignIds(self, figure):
        self.curid = 0
        self._walkTreeAssign(figure._children)
    
    def _walkTreeAssign(self, children):
        """ The walker to assign ids to all objects.
        This walker walks in the same order as the walker that
        searches for the objec given an id. This way we do not
        have to test each object, but only go down the branches
        where it is in. """
        for child in children:
            id = self.GetId()
            child._id = id
            # proceed to children
            if hasattr(child,'_wobjects'):
                self._walkTreeAssign(child._wobjects)
            self._walkTreeAssign(child._children)
    
    def _walkTree(self, id, children, items):
        """ The walker. """
        for i in range(len(children)):
            # define child and next child
            child = children[i]
            child2 = None
            if i+1 < len(children):
                child2 = children[i+1]
            # compare id's
            if id == child._id:
                # found it!
                items.append(child)
                break
            elif id > child._id:
                if not child2:
                    # last child
                    items.append(child)
                    if hasattr(child,'_wobjects'):
                        self._walkTree(id, child._wobjects, items)
                    self._walkTree(id, child._children, items)
                elif id < child2._id:
                    # the child is a subchild!
                    items.append(child)
                    if hasattr(child,'_wobjects'):
                        self._walkTree(id, child._wobjects, items)
                    self._walkTree(id, child._children, items)
                else:
                    continue



class BaseFigure(_BaseFigure):
    """ BaseFigure()
    
    Abstract class representing the root of all wibjects.
    
    A Figure is a wrapper around the OpenGL widget in which it is drawn;
    this way different backends are possible. Each backend inherits this
    class and implements the required methods and makes sure all GUI
    events are translated to visvis events.
    
    Since the figure represents the OpenGl context and is the root
    of the visualization tree; a Figure Wibject does not have a parent.
    
    A Figure can be created with the function vv.figure() or vv.gcf().
    
    """
    
    # dictionary of all figures objects: int -> Figure instance
    _figures = {}
    # the current figure object
    _currentNr = None
    
    def __init__(self):
        """ The init will be oveloaded in the subclasses,
        but they should call this init from there.
        """
        base.Wibject.__init__(self, None)
        
        # register
        self._Register()
        
        # Make sure we have _devicePixelRatio
        if not hasattr(self, "_devicePixelRatio"):
            self._devicePixelRatio = 1.0
        
        # to prevent recursion
        self._resizing = False
        
        # create a timer to handle the drawing better
        self._drawTimer = events.Timer(self, 10, oneshot=True)
        self._drawTimer.Bind(self._DrawTimerTimeOutHandler)
        self._drawtime = time.time() # to calculate fps
        self._drawWell = -1
        
        # init background
        self.bgcolor = 0.8,0.8,0.8  # bgcolor is a property
        
        # Location of the mouse (in pixel coordinates).
        # It is the implementing class' responsibility to
        # continously updated this.
        self._mousepos = (0,0)
        
        # The title
        self._title = ''
        
        # Each figure directly corresponds to an openGL context.
        # Therefore figure instances function as a place for
        # keeping objects that should exist per context...
        
        # For keeping track of mouse and picking.
        self._pickerHelper = ObjectPickerHelper()
        self._underMouse = []
        
        # To store the fonts used in this figure.
        self._fontManager = visvis.text.FontManager()
        self._relativeFontSize = visvis.settings.defaultRelativeFontSize
        
        # Whether to enable user interaction
        self._enableUserInteraction = True
        
        # To store the markers used in this figure
        self._markerManager = MarkerManager()
        
        # keep track of the currently active axes of this figure.
        self._currentAxes = None
        
        # Create events that only the figure has
        self._eventClose = events.BaseEvent(self)
        self._eventAfterDraw = events.BaseEvent(self)
        
        # Bind to events
        self.eventPosition.Bind(self._OnPositionChange)
    
    
    @property
    def eventClose(self):
        """ Fired when the figure is closed.
        """
        return self._eventClose
    
    @property
    def eventAfterDraw(self):
        """ Fired after each drawing pass.
        """
        return self._eventAfterDraw
    
    
    def _Register(self):
        """ _Register()
        
        Register the figure with the list of figures.
        
        """
        
        # get keys
        nrs = list(BaseFigure._figures.keys())
        nrs.sort()
        nr = 0 # the number...
        # check if a spot was prepared for us (override if so)
        for i in nrs:
            if BaseFigure._figures[i] is None:
                nr = i
                break
        # should we try finding our own number?
        if not nr:
            # empty?
            if not nrs:
                nrs.append(0)
            # check if there is a spot free
            for i in range(1,len(nrs)+1):
                if not i==nrs[i-1]:
                    nr = i
                    break
            else:
                # no, append to the end
                nr = nrs[-1]+1
        # set
        BaseFigure._figures[nr] = self
        # make current
        BaseFigure._currentNr = nr
    
    
    def _dpi_aware_viewport(self, x, y, w, h):
        pr = self._devicePixelRatio
        gl.glViewport(int(x * pr), int(y * pr), int(w * pr), int(h * pr))
    
    def _normal_ortho(self, *args):
        # For when drawing in screen coords
        ortho(*args)
    
    def _dpi_aware_ortho(self, *args):
        # For when drawing in screen coords
        pr = self._devicePixelRatio
        #gl.glScale(pr, pr, 1.0)
        args = [a * pr for a in args]
        ortho(*args)
        
    ## Methods to overload
    # To create a subclass for a specific backend:
    # - Overload the methods below.
    # - Make sure Draw() is called on each paint request.
    # - Pass the events on and keep visvis timers running.
    
    def _SetCurrent(self):
        """ _SetCurrent()
        
        Make the figure the current OpenGL context. This is required before
        drawing and before doing anything with OpenGl really.
        
        """
        raise NotImplementedError()
    
    def _SwapBuffers(self):
        """ _SwapBuffers()
        
        Swap the memory and screen buffer such that
        what we rendered appears on the screen.
        
        """
        raise NotImplementedError()
    
    def _ProcessGuiEvents(self):
        """ _ProcessGuiEvents()
        
        Process all events in the event queue.
        This is usefull when calling Draw() while an algorithm is
        running. The figure is then still responsive.
        
        """
        raise NotImplementedError()
        
    def _SetTitle(self, title):
        """ _SetTitle(title)
        
        Set the title of the figure. Note that this
        does not have to work if the Figure is used as
        a widget in an application.
        
        """
        raise NotImplementedError()
    
    def _SetPosition(self, x, y, w, h):
        """ _SetPosition(x, y, w, h)
        
        Set the position of the widget. The x and y represent the
        screen coordinates when the figure is a toplevel widget.
        
        For embedded applications they represent the position within
        the parent widget.
        
        """
        raise NotImplementedError()
    
    def _GetPosition(self):
        """ _GetPosition()
        
        Get the position of the widget. The result must be a
        four-element tuple (x,y,w,h).
        
        """
        raise NotImplementedError()
    
    
    def _Close(self):
        """ _Close()
        
        Close the widget, also calls Destroy().
        
        """
        raise NotImplementedError()
    
    ## Properties
    
    @PropWithDraw
    def devicePixelRatio():
        """ The scale factor for high-DPI displays. This value is usually
        1 (for normal display) or 2 (for retina displays). When using a Qt-based
        backend, this value will usually be set correctly.
        """
        def fget(self):
            return self._devicePixelRatio
        def fset(self, value):
            self._devicePixelRatio = float(value)
        return locals()
    
    @Property
    def parent():
        """ The parent of a figure always returns None and cannot be set.
        """
        def fget(self):
            return None
        def fset(self, value):
            pass
            #raise AttributeError("The parent of a figure cannot be set.")
            # do not raise an exception, as the parent is always ste in
            # the constructor of the BaseObject.
        return locals()
    
    @property
    def nr(self):
        """ Get the number (id) of this figure.
        """
        for key in BaseFigure._figures:
            if BaseFigure._figures[key] is self:
                return key
    
    @PropWithDraw
    def title():
        """ Get/Set the title of the figure. If an empty string or None,
        will display "Figure X", with X the figure nr.
        """
        def fget(self):
            return self._title
        def fset(self, value):
            if value:
                value = str(value)
                self._SetTitle(value)
            else:
                value = ''
                self._SetTitle("Figure "+str(self.nr))
            self._title = value
        return locals()
    
    
    def MakeCurrent(self):
        """ MakeCurrent()
        
        Make this the current figure.
        Equivalent to "vv.figure(fig.nr)".
        
        """
        BaseFigure._currentNr = self.nr
    
    
    @PropWithDraw
    def currentAxes():
        """ Get/Set the currently active axes of this figure.
        Returns None if no axes are present.
        """
        def fget(self):
            
            # init
            curNew = None
            curOld = self._currentAxes
            if curOld:
                curOld = curOld() # because is weak ref
            
            # check
            for child in self._children:
                if not isinstance(child, AxesContainer):
                    continue
                child = child.GetAxes()
                if not child:
                    continue
                if not curNew:
                    curNew = child
                if child is curOld:
                    curNew = child
                    break
            
            # update and return
            if curNew:
                self._currentAxes = curNew.GetWeakref()
            else:
                self._currentAxes = None
            return curNew
        
        def fset(self, value):
            if value is None:
                self._currentAxes = None
            elif isinstance(value, Axes):
                self._currentAxes = value.GetWeakref()
            else:
                raise ValueError('currentAxes must be an Axes instance (or None).')
        
        return locals()
    
    
    @property
    def underMouse(self):
        """ Get the object currently under the mouse. Can be None.
        """
        if not self._underMouse:
            return None
        return self._underMouse[-1]()
    
    @property
    def mousepos(self):
        """ Get the position of the mouse in figure coordinates.
        """
        return self._mousepos
    
    # ===== Notes about positioning figures.  =====
    # Moving a figure by dragging it with the mouse or programatically
    # using the undelying widget directly does not fire a position event.
    # On resizing it does, because all backends call _OnResize() when this
    # happens. Setting the position using the position property simply
    # updates the position, which will make the eventPosition being fired
    # which makes the bound _OnPositionChange() being called, which updates
    # the actual position and or size of the underlying widget (via the
    # backend-implemented _SetPosition().
    
    @Property # Moving a figure always invokes an update from the window manager
    def position():
        """ The position for the figure works a bit different than for
        other wibjects: it only works with absolute values and it
        represents the position on screen or the position in the
        parent widget in an application.
        """
        def fget(self):
            # Update the position by asking the backend
            # Note we need to use privates to avoid position.Update() being
            # called
            thepos = self._GetPosition()
            self._position._x, self._position._y = thepos[0], thepos[1]
            self._position._w, self._position._h = thepos[2], thepos[3]
            self._position._inpixels = tuple(thepos)
            self._position._absolute = tuple(thepos)
            return self._position
        
        def fset(self,value):
            # check input
            value = [int(v) for v in value]
            if len(value) not in [2,4]:
                raise ValueError("Position must consist of 2 or 4 elements.")
            for v in value[2:]:
                if v < 2:
                    raise ValueError(   "Figure heigh and weight " +
                                        "must be given in absolute pixels.")
            # allow setting x and y only
            if len(value) == 2:
                value = value[0], value[1], self._position.w, self._position.h
            # make pos
            self._position = base.Position( value[0], value[1],
                                            value[2], value[3], self)
            self._position._Changed()
        
        return locals()
    
    
    def _OnPositionChange(self,event=None):
        """ _OnPositionChange(event=None)
        
        When the position was programatically changed, we should
        change the position of the window.
        But ONLY if it was really changed (or we get into infinite loops).
        
        """
        
        #if self._resizing:
        #    return
        pos1 = self._GetPosition()
        pos2 = tuple( [int(i) for i in self._position] )
        if pos1 != pos2:
            self._SetPosition( *pos2 )
    
    
    def _OnResize(self, event=None):
        """ _OnResize(event=None)
        
        Called when the figure is resized.
        This should initiate the event_position event, but not by firing
        the event_position of this object, otherwise it is not propagated.
        
        """
        # Allow position tree to update
        self.position._Changed()
        # Draw, but not too soon.
        self.Draw(timeout=200)
    
    
    @PropWithDraw
    def relativeFontSize():
        """ The (global) relative font size; all texts in this figure
        are scaled by this amount. This is intended to (slighly) increase
        or descrease font size in the figure for publication purposes.
        """
        def fget(self):
            return self._relativeFontSize
        def fset(self, value):
            # Set value
            self._relativeFontSize = float(value)
            # Update all text objects
            for ob in self.FindObjects(BaseText):
                ob.Invalidate()
            # Update all legend objects
            for ob in self.FindObjects(Legend):
                ob.SetStrings(ob._stringList)
        return locals()
    
    @Property
    def enableUserInteraction():
        """ Whether to allow user interaction. The default is True. This
        property can be set to False to improve performance (expensive
        calls to glDrawPixels can be omitted).
        """
        def fget(self):
            return self._enableUserInteraction
        def fset(self, value):
            self._enableUserInteraction = bool(value)
        return locals()
    
    ## Extra methods
    
    @DrawAfter
    def Clear(self):
        """ Clear()
        
        Clear the figure, removing all wibjects inside it and clearing all
        callbacks.
        
        """
        # remove children
        while self._children:
            child = self._children.pop()
            if hasattr(child, 'Destroy'):
                child.Destroy()
        # remove callbacks
        for fieldName in self.__dict__:
            if fieldName.startswith('_event'):
                event = self.__dict__[fieldName]
                if hasattr(event,'Unbind'):
                    event.Unbind()
        # Restore some event bindings
        self.eventPosition.Bind(self._OnPositionChange)
    
    ## Implement methods
    
    
    def Destroy(self):
        """ Destroy()
        
        Close the figure and clean up all children.
        
        """
        if self._widget is None:
            #raise RuntimeError('Attempt to Destroy a dead Figure.')
            # Do not raise error, because we make Wibject.__del__ call Destroy
            return
        
        # Fire event to notify closing while everything is still intact
        self.eventClose.Fire()
        
        # Clean up
        base.Wibject.Destroy(self)
        
        # Detach reference to backend widget. We need to keep the widget
        # alive because we are not allowed to destroy it during its
        # callback.
        # So we place a reference somewhere, meaning that the widget (and
        # thus the Figure instance can be cleaned up after a second figure
        # has been destroyed. In other words, there will never be more than
        # one destroyed alive Figure instance (if the user cleans up its
        # references).
        # Note that WX really removes the widget and replaces it with a
        # dummy wrapper.
        BaseFigure._lastClosedWidget = w = self._widget
        self._widget = None
        w.figure = None
        
        # Close widget
        self._Close(w)
    
    
    def OnDestroy(self):
        # remove from list
        for nr in list(BaseFigure._figures.keys()):
            if BaseFigure._figures[nr] is self:
                BaseFigure._figures.pop(nr)
                BaseFigure._currentFigure = None
                break
    
    
    def Draw(self, fast=False, timeout=10):
        """ Draw(fast=False, timeout=10)
        
        Draw the figure within 10 ms (if the events are handled).
        Multiple calls in a short amount of time will result in only
        one redraw.
        
        """
        
        # Only if not currently being drawn
        if self._isbeingdrawn:
            return False
        
        # Get whether the request is to draw well
        well = not fast
        
        # Set draw well (is reciprocal of fast, to handle it easier.)
        # -1 means it is unset, and when a paint event occurs, will draw well
        # 0 means to draw fast, will only occur if only Draw(True) calls
        # 1 means draw well.
        if self._drawWell == -1:
            self._drawWell = well
        else:
            self._drawWell = self._drawWell or well
        
        # Restart timer if we need to
        if not self._drawTimer.isRunning:
            self._drawTimer.Start(timeout)
        
        # Done
        return True
    
    
    def _DrawTimerTimeOutHandler(self, event=None):
        self._RedrawGui() # post event
    
    
    def DrawNow(self, fast=False):
        """ DrawNow(fast=False)
        
        Draw the figure right now and let the GUI toolkit process its events.
        Call this from time to time if you want to update your figure while
        running some algorithm, and let the figure stay responsive.
        
        """
        self._drawWell = not bool(fast)
        self._RedrawGui() # post event
        self._ProcessGuiEvents() # process all events (including our draw)
    
    
    def OnDraw(self, event=None):
        """ OnDraw()
        
        Perform the actual drawing. Called by the GUI toolkit paint event
        handler. Users should not call this method, but use
        Draw() or DrawNow().
        
        """
        # This is the actual draw entry point. But we will
        # call _Draw() to draw first the beatiful pictures and then
        # again to only draw the shapes for picking.
        
        # are we alive?
        if self._destroyed:
            return
        
        # Init drawing
        self._isbeingdrawn = True
        
        try:
            # calculate fps
            dt = time.time() - self._drawtime
            self._drawtime = time.time()
            if printFPS:
                print('FPS: %1.1f' % (1.0/dt))  # for testing
            
            # get whether to draw fast and reset drawWell flag
            fast = self._drawWell == 0
            self._drawWell = -1
            
            # make sure to draw to this canvas/widget
            self._SetCurrent()
            
            # get bits for this buffer
            self._pickerHelper.bits_r = rb = gl.glGetIntegerv(gl.GL_RED_BITS)
            self._pickerHelper.bits_g = gb = gl.glGetIntegerv(gl.GL_GREEN_BITS)
            self._pickerHelper.bits_b = bb = gl.glGetIntegerv(gl.GL_BLUE_BITS)
            if 0 in [rb, gb, bb]:
                raise RuntimeError('OpenGL context not set.')
            
            # set ids
            self._pickerHelper.AssignIds(self)
            
            # draw shape (to backbuffer)
            if self._enableUserInteraction:
                self._Draw(DRAW_SHAPE)
                gl.glFinish() # call finish, normally swapbuffers does this...
            
            # read screen (of backbuffer)
            # We can improve performance if we do not capture the screen here
            if self._enableUserInteraction:
                self._pickerHelper.CaptureScreen()
            else:
                self._pickerHelper.ClearScreen()
            #self._SwapBuffers() # uncomment to see the color coded objects
            
            # draw picture
            mode = [DRAW_NORMAL, DRAW_FAST][bool(fast)]
            self._Draw(mode)
            
            # write the output to the screen
            self._SwapBuffers()
            
            # Notify
            self.eventAfterDraw.Fire()
        
        finally:
            self._isbeingdrawn = False
    
    
    def _Draw(self, mode):
        """ _Draw(mode)
        
        This method performs a single drawing pass. Used by OnDraw().
        
        """
        
        # make sure the part to draw to is ok
        w,h = self.position.size
        
        # clear screen
        
        # init pickerhelper as one
        pickerHelper = None
        
        if mode==DRAW_SHAPE:
            # clear
            clr = (0,0,0)
            gl.glClearColor(0.0 ,0.0 ,0.0, 0.0)
            gl.glClear(gl.GL_COLOR_BUFFER_BIT | gl.GL_DEPTH_BUFFER_BIT)
            
            # do not blend not light
            gl.glDisable(gl.GL_BLEND)
            
            # nor smooth
            gl.glDisable(gl.GL_POINT_SMOOTH)
            gl.glDisable(gl.GL_LINE_SMOOTH)
            
            # get pickerhelper
            pickerHelper = self._pickerHelper
        
        else:
            # clear
            clr = self.bgcolor
            gl.glClearColor(clr[0], clr[1], clr[2], 0.0)
            gl.glClear(gl.GL_COLOR_BUFFER_BIT | gl.GL_DEPTH_BUFFER_BIT)
            
            # Set lighting model properties.
            # - We do not use the global ambient term
            # - We do not use local viewer mode
            # - We want to allow people to see also backfaces correctly
            # - We want to be texture-proof for specular highlights
            # Note: Individual lights are set in the camera class.
            glVersion = getOpenGlInfo()[0]
            gl.glLightModelfv(gl.GL_LIGHT_MODEL_AMBIENT, (0,0,0,1))
            gl.glLightModelf(gl.GL_LIGHT_MODEL_LOCAL_VIEWER, 0.0)
            gl.glLightModelf(gl.GL_LIGHT_MODEL_TWO_SIDE, 1.0)
            if glVersion >= '1.2':
                gl.glLightModelf(gl.GL_LIGHT_MODEL_COLOR_CONTROL,
                    gl.GL_SEPARATE_SPECULAR_COLOR)
            
            # enable blending, so lines and points can be antialiased
            gl.glEnable(gl.GL_BLEND)
            gl.glBlendFunc(gl.GL_SRC_ALPHA, gl.GL_ONE_MINUS_SRC_ALPHA)
            #gl.glEnable(gl.GL_MULTISAMPLE)
            # smooth lines
            gl.glEnable(gl.GL_LINE_SMOOTH)
            if mode==DRAW_FAST:
                gl.glHint(gl.GL_POINT_SMOOTH_HINT, gl.GL_FASTEST)
                gl.glHint(gl.GL_LINE_SMOOTH_HINT, gl.GL_FASTEST)
            else:
                gl.glHint(gl.GL_POINT_SMOOTH_HINT, gl.GL_NICEST)
                gl.glHint(gl.GL_LINE_SMOOTH_HINT, gl.GL_NICEST)
        
        
        # can we draw at all?
        if not (w>0 and h>0):
            return
        
        # Draw axes. Each axes specifies the viewport for itself and draws
        # all wobjects in it. Then it prepares for the wibjects, which are
        # drawn (via _DrawTree) in the same viewport.
        for child in self._children:
            if isinstance(child, (Axes, AxesContainer) ):
                self._PrepateForFlatDrawing(w, h)
                child._DrawTree(mode, pickerHelper)
        
        # Draw other children
        self._PrepateForFlatDrawing(w, h)
        for child in self._children:
            if not isinstance(child, (Axes, AxesContainer)):
                child._DrawTree(mode, pickerHelper)

    
    def _PrepateForFlatDrawing(self, w, h):
        
        # Set viewport
        gl.glDisable(gl.GL_DEPTH_TEST)
        self._dpi_aware_viewport(0, 0, w, h)
        
        # set camera
        # Flip up down because we want y=0 to be on top.
        gl.glMatrixMode(gl.GL_PROJECTION)
        gl.glLoadIdentity()
        self._normal_ortho(0, w, h, 0)
        gl.glMatrixMode(gl.GL_MODELVIEW)
        gl.glLoadIdentity()
    
    
    def _GenerateKeyEvent(self, eventName, *eventArgs):
        """ _GenerateKeyEvent(eventName, *eventArgs)
        
        For the backend to generate key events.
        
        """
        if not self._enableUserInteraction:
            return
        
        # make lower
        eventName = eventName.lower()
        # get items now under the mouse
        items1 = [item() for item in self._underMouse if item()]
        # init list of events to fire
        events = []
        eventToAlwaysFire = None
        
        # Check what to fire
        if eventName.count('down'):
            eventToAlwaysFire = self.eventKeyDown
            if items1:
                events.append(items1[-1].eventKeyDown)
        elif eventName.count('up'):
            eventToAlwaysFire = self.eventKeyUp
            if items1:
                events.append(items1[-1].eventKeyUp)
        
        # Fire events. We use an approach to ensure that the key event
        # is always fired for the Figure, but only once, not if that event
        # was fired because it propagated from its children.
        if eventToAlwaysFire:
            eventToAlwaysFire._isFired = False
        for event in events:
            event.Fire(*eventArgs)
        if eventToAlwaysFire and not eventToAlwaysFire._isFired:
            eventToAlwaysFire.Fire(*eventArgs)
    
    
    def _GenerateMouseEvent(self, eventName, *eventArgs):
        """ _GenerateMouseEvent(eventName, *eventArgs)
        
        For the backend to generate mouse events.
        
        """
        if not self._enableUserInteraction:
            return
        
        # make lower
        eventName = eventName.lower()
        # get items now under the mouse
        items1 = [item() for item in self._underMouse if item()]
        # init list of events to fire
        events = []
        
        if eventName.count("motion") or eventName.count("move"):
            
            # Set current mouse pos, so that it's up to date
            self._mousepos = eventArgs[0], eventArgs[1]
            
            # also get new version of items under the mouse
            items2 = self._pickerHelper.GetItemsUnderMouse(self)
            
            # init event list
            events = []
            
            # analyse for enter and leave events
            for item in items1:
                if item not in items2:
                    events.append(item.eventLeave)
            for item in items2:
                if item not in items1:
                    events.append(item.eventEnter)
            
            # Generate motion events for any objects that have handlers
            # for the motion event. Note that this excludes "self".
            items = self.FindObjects(lambda i:i.eventMotion.hasHandlers)
            events.extend([item.eventMotion for item in items])
            
            # Always generate motion event from figure
            events.append(self.eventMotion)
            
            # Update items under the mouse
            self._underMouse = [item.GetWeakref() for item in items2]
            
            # Note: how to handle motionEvents
            # It feels nicer (and is more Qt-ish) to fire the events only
            # from the objects that have their mouse pressed down. However,
            # someone may want to be notified of motion regardless of the
            # state of the mouse. Qt has setMouseTracking(True).
            # But then there's a problem. This version of eventMotion we *do*
            # want to propate to parent objects if not handled. But then
            # an object may get an event twice if the mouse is pressed down.
            # Surely these things can be overcome, but I think its easier
            # for everyone if we stick to the old approach: the eventMotion
            # is fired for all objects that have handlers registered to it.
        
        elif eventName.count("up"):
            # Find objects that are currently clicked down
            items = self.FindObjects(lambda i:i._mousePressedDown)
            if self._mousePressedDown: items.append(self)
            events.extend([item.eventMouseUp for item in items])
            # todo: Also generate event where the mouse is now over - MouseDrop?
            #if items[-1] not in items:
            #    events.append( items[-1].eventMouseDrop)
        
        elif items1 and eventName.count("down"):
            events.append(items1[-1].eventMouseDown)
        
        elif items1 and eventName.count("double"):
            # Note: we cannot detect double clicking by timing the down-events,
            # because the toolkit won't fire a down event for the second click.
            events.append(items1[-1].eventDoubleClick)
        
        elif items1 and eventName.count('scroll'):
            events.append(items1[-1].eventScroll)
        
        # Fire events
        for event in events:
            event.Fire(*eventArgs)



#!/usr/bin/env python
""" This example shows hoe the GLSL program for a 2D texture
can be modified to achieve all kinds of graphic effects.
In this example we will demonstrate sharpening an image using
unsharp masking.
"""

import visvis as vv

# First define our part of the shading code
# The '>>' Denote what piece of code we want to replace.
# There is a pre-loop section, that is executed before the anti-aliasing
# loop. We use this to modify the aliasing kernel so it always does a fixed
# amount of smoothing.
# In the post-loop section we combine the subtracted image with the normal
# sampled image. Unsharp masking consists of subtracting the smoothed image
# from the original (thus removing low frequency components), and then adding
# the result with a certain factor to the original.
#
# Note that the aa kernel is symetric; kernel[0] is the center pixel, and
# kernel[1] through kernel[3] is the tail on all ends.
SH_2F_SHARPEN = vv.shaders.ShaderCodePart('sharpen','unsharp masking',
"""
    >>--uniforms--
    uniform float amount;
    // --uniforms--
    
    >>--pre-loop--
    sze = 3; // Use full kernel (otherwise it wont work if t.aa == 0)
    kernel = vec4(1.0, 0.9, 0.6, 0.3); // approximate Gauss of sigma 2
    float kernel_norm = kernel[0] + (kernel[1] + kernel[2] + kernel[3])*2.0;
    kernel /= kernel_norm;
    // --pre-loop--
    
    >>--post-loop--
    float th = 0.05;
    vec4 normalColor = texture2D(texture, pos);
    // Element-wise mask on blurred image (color1), using a threshold
    float mask = float(length(color1.rgb)-length(color2.rgb)>th);
    normalColor.rgb += mask * amount * (normalColor.rgb -color1.rgb);
    color1 = normalColor;
    // --post-loop--
""")

# Read image
im = vv.imread('astronaut.png')

# Show two times, the second will be sharpened
vv.subplot(121); t1 = vv.imshow(im)
vv.subplot(122); t2 = vv.imshow(im)

# Share cameras and turn off anti-aliasing for proper comparison
t1.parent.camera = t2.parent.camera
t1.aa = 0

# Insert our part in the fragment shader program
t2.shader.fragment.AddOrReplace(SH_2F_SHARPEN, after='base')
if False: # Execute this line to turn it off:
    t2.shader.fragment.RemovePart('sharpen')

# Make a slider to set the amount
def sliderCallback(event):
    t2.shader.SetStaticUniform('amount', slider.value)
    t2.Draw()
slider = vv.Slider(t2.parent, (0.0, 1.5))
slider.textColor = 'blue'
slider.position = 0.05, 10, 0.9, 40
slider.eventSliding.Bind(sliderCallback)
sliderCallback(None) # init uniform

# In case there are bugs in the code, it might be helpfull to see the code
# t2.fragmentShader.ShowCode() # Shows the whole code
t2.shader.fragment.ShowCode('sharpen') # Shows only our bit, with line numbers

# Run app
app = vv.use()
app.Run()



# -*- coding: utf-8 -*-
# flake8: noqa
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

""" Module wibjects

All wibjects are inserted in this namespace, thereby providing
the user with a list of all wibjects. All wibjects are also
inserted in the root visvis namespace.

"""

from visvis.core import (  Wibject, BaseFigure, AxesContainer, Axes,
                           Box, DraggableBox, Label, Legend,
                        )

from visvis.wibjects.buttons import PushButton, ToggleButton, RadioButton
from visvis.wibjects.sliders import BaseSlider, Slider, RangeSlider
from visvis.wibjects.colorWibjects import (BaseMapableEditor, Colorbar,
                                    ColormapEditor, ClimEditor)
from visvis.wibjects.title import Title



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import visvis as vv

def movieShow(images, clim=None, duration=0.1, axesAdjust=True, axes=None):
    """ movieShow(images, duration=0.1)
    
    Show the images in the given list as a movie.
    
    Parameters
    ----------
    images : list
        The 2D images (can be color images) that the movie consists of.
    clim : (min, max)
        The color limits to apply. See imshow.
    duration : scalar
        The duration (in seconds) of each frame. The real duration can
        differ from the given duration, depending on the performance of
        your system.
    axesAdjust : bool
        If axesAdjust==True, this function will call axes.SetLimits(), set
        the camera type to 2D, and make axes.daspect[1] negative (i.e. flip
        the y-axis). If daspectAuto has not been set yet, it is set to False.
    axes : Axes instance
        Display the image in this axes, or the current axes if not given.
    
    """
    
    # Get axes
    if axes is None:
        axes = vv.gca()
    
    # Create container
    m = vv.MotionDataContainer(axes, duration*1000)
    
    # Create images and put in container
    for im in images:
        t = vv.imshow(im, clim=clim, axesAdjust=axesAdjust, axes=axes)
        t.parent = m
    
    # Return container object
    return m


if __name__ == '__main__':
    ims = vv.movieRead('newtonscradle.gif')
    vv.movieShow(ims)




def test_visvis_import():
    import visvis as vv
    import visvis.vvio
    
    assert vv
    assert vv.vvio
    assert vv.imread
    assert vv.imshow
    assert vv.plot
    assert vv.Slider
    assert vv.Axes
